import base64

from click.testing import CliRunner

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.b64 import d
from soh.b64 import e


def test_b64e(sample_text):
    runner = CliRunner()

    # build args
    args = [sample_text]

    result = runner.invoke(e, args)

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    assert output == base64.b64encode(sample_text.encode("utf-8")).decode("utf-8")

    return result


def test_b64d(sample_b64, pad):
    runner = CliRunner()

    # build args
    args = [sample_b64]
    if pad is not None:
        args += [pad]

    result = runner.invoke(d, args)

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    assert output == base64.b64decode(sample_b64.encode("utf-8")).decode("utf-8")

    return result



"""Utility functions and decorators."""
from functools import wraps

import click
import pyperclip
import requests


COPIED_TO_CLIPBOARD_MESSAGE = " (copied to clipboard 📋)"


def clipboard_output(func):
    @click.option("-c", "--clip", is_flag=True, help="copy to clipboard")
    @wraps(func)
    def handle_output(*args, **kwargs):
        clip = kwargs.pop("clip", False)
        protect = kwargs.pop("protect", False)
        value = func(*args, **kwargs)
        if clip:
            if value != "" and not protect:
                click.secho(value, fg="yellow", nl=False)
            click.secho(COPIED_TO_CLIPBOARD_MESSAGE, fg="green")
            pyperclip.copy(value)
        else:
            if value != "" and not protect:
                click.secho(value)

    return handle_output


def ensure_ok_response(response, message):
    try:
        response.raise_for_status()
    except requests.RequestException:
        raise click.ClickException(message)



import ipaddress
import multiprocessing
import platform
import socket
from uuid import getnode

import vcr
from click.testing import CliRunner

from .common import compare
from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.system import all_
from soh.system import arch
from soh.system import cores
from soh.system import eip
from soh.system import ip
from soh.system import mac
from soh.system import machine
from soh.system import node
from soh.system import platform_
from soh.system import proc
from soh.system import version
from .test_util import check_clipboard_output


def test_all():
    runner = CliRunner()
    args = []

    result = runner.invoke(all_, args)

    assert result.exit_code == 0


# @check_clipboard_output
def test_arch():  # , clip):
    args = []
    compare(arch, args, "; ".join(platform.architecture()))


# @check_clipboard_output
def test_cores():  # , clip):
    args = []
    compare(cores, args, str(multiprocessing.cpu_count()))


# @check_clipboard_output
def test_eip():  # , clip):
    args = []
    with vcr.use_cassette("tests/cassettes/system_eip.yaml"):
        result = compare(eip, args)
    value = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    address = ipaddress.ip_address(value)
    assert address.is_global


# @check_clipboard_output
def test_ip():  # , clip):
    args = []
    result = compare(ip, args, socket.gethostbyname(socket.gethostname()))
    value = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    address = ipaddress.ip_address(value)
    assert address.is_private


# @check_clipboard_output
def test_mac(upper):  # , clip):
    args = []
    if upper is not None:
        args += [upper]

    compare_to = ":".join(("%012x" % getnode())[i : i + 2] for i in range(0, 12, 2))
    if upper:
        compare_to = compare_to.upper()

    compare(mac, args, compare_to)


# @check_clipboard_output
def test_machine():  # , clip):
    args = []
    compare(machine, args, platform.machine())


# @check_clipboard_output
def test_node():  # , clip):
    args = []
    compare(node, args, platform.node())


# @check_clipboard_output
def test_platform():  # , clip):
    args = []
    compare(platform_, args, platform.system())


# @check_clipboard_output
def test_proc():  # , clip):
    args = []
    compare(proc, args, platform.processor())


# @check_clipboard_output
def test_version():  # , clip):
    args = []
    compare(version, args, platform.version())



from click.testing import CliRunner

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.uuid import uuid_
from .test_util import check_clipboard_output


# @check_clipboard_output
def test_uuid(
    uuid_version_label, uuid_version, uuid_namespace_label, uuid_namespace, uuid_name_label, uuid_name, upper  # clip
):
    runner = CliRunner()

    # build args
    args = []
    if uuid_version is not None:
        args += [uuid_version_label, uuid_version]
    if uuid_namespace is not None:
        args += [uuid_namespace_label, uuid_namespace]
    if uuid_name is not None:
        args += [uuid_name_label, uuid_name]
    if upper is not None:
        args += [upper]
    # if clip is not None:
    #     args += [clip]

    result = runner.invoke(uuid_, args)

    # invalid combinations
    if uuid_version in (3, 5) and (
        uuid_namespace is None or uuid_namespace.lower() not in ("dns", "url", "oid", "x500")
    ):
        assert result.exit_code != 0
    elif uuid_version not in (3, 5) and uuid_namespace is not None:
        assert result.exit_code != 0
    elif uuid_version not in (None, 1, 3, 4, 5):
        assert result.exit_code != 0
    else:
        assert result.exit_code == 0

    if upper is not None and result.exit_code == 0:
        msg = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0]
        assert msg == msg.upper()

    return result



import pytest


# clipboard
@pytest.fixture(params=[None, "-c", "--clip"])
def clip(request):
    return request.param


# uuid
@pytest.fixture(params=[None, 1, 2, 3, 4, 5])
def uuid_version(request):
    return request.param


@pytest.fixture(params=["-v", "--version"])
def uuid_version_label(request):
    return request.param


@pytest.fixture(params=[None, "dns", "url", "oid", "x500", "invalid"])
def uuid_namespace(request):
    return request.param


@pytest.fixture(params=["-ns", "--namespace"])
def uuid_namespace_label(request):
    return request.param


@pytest.fixture(params=[None, "something"])
def uuid_name(request):
    return request.param


@pytest.fixture(params=["-n", "--name"])
def uuid_name_label(request):
    return request.param


# upper case results
@pytest.fixture(params=[None, "-u", "--upper"])
def upper(request):
    return request.param


# base 64
@pytest.fixture(params=["something"])
def sample_text(request):
    return request.param


# "something" b64encoded
@pytest.fixture(params=["c29tZXRoaW5n"])
def sample_b64(request):
    return request.param


@pytest.fixture(params=[None, "-p", "--pad"])
def pad(request):
    return request.param


# epoch
@pytest.fixture(params=[None, "-f", "--float"])
def float_(request):
    return request.param


# serve
@pytest.fixture(params=[None, "-p", "--port"])
def port_label(request):
    return request.param


# datetime
@pytest.fixture(params=[None, "-t"])
def t(request):
    return request.param


@pytest.fixture(params=[None, "-tz"])
def tz_label(request):
    return request.param


@pytest.fixture(params=["list", "EST", "not_a_timezone"])
def tz(request):
    return request.param


# json
@pytest.fixture(params=[None, "-i", "--indent"])
def indent_label(request):
    return request.param


@pytest.fixture(params=[2, 3, 5])
def indent(request):
    return request.param


@pytest.fixture(params=[None, "-a", "--ascii"])
def ascii_(request):
    return request.param


json_sample = """'{"a":"b","c":1,"d":{"e":"f"}"g":[1,2,3],"s":"♣"}'"""
json_sample_bad = """'{"a":"b","c":1,"d":{"e":"f"}"g":[1,2,3],"s":"♣"'"""


@pytest.fixture(params=[json_sample, json_sample_bad])
def json_text(request):
    return request.param


# create
@pytest.fixture(params=[None, "-o", "--overwrite"])
def overwrite(request):
    return request.param


@pytest.fixture(params=["python", "list", "invalid_language"])
def language(request):
    return request.param


@pytest.fixture(params=["mit", "list", "invalid_license"])
def value(request):
    return request.param


# tokens
@pytest.fixture(
    params=[
        "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c"
    ]
)
def jwt_sample(request):
    return request.param


# secret
@pytest.fixture(params=[None, "-b", "--bytes"])
def nbytes_label(request):
    return request.param


@pytest.fixture(params=[16, 64])
def nbytes(request):
    return request.param


@pytest.fixture(params=[None, "-n"])
def qty_num_label(request):
    return request.param


@pytest.fixture(params=[0, 24])
def qty_num(request):
    return request.param


@pytest.fixture(params=[None, "-s"])
def qty_sym_label(request):
    return request.param


@pytest.fixture(params=[0, 24])
def qty_sym(request):
    return request.param


@pytest.fixture(params=[None, "-l"])
def length_label(request):
    return request.param


@pytest.fixture(params=[8, 64])
def length(request):
    return request.param


@pytest.fixture(params=[None, "-a"])
def allow_ambiguous(request):
    return request.param


@pytest.fixture(params=[None, "-p"])
def protect(request):
    return request.param



"""CLI functionality.

Entry point: $ soh --help
Entry point: $ soh version
"""
import click

from soh.b64 import b64
from soh.create import create
from soh.datetime import dt
from soh.epoch import epoch
from soh.serve import serve
from soh.json import json_
from soh.secret import secret
from soh.system import sys
from soh.tokens import jwt
from soh.uuid import uuid_
from soh.util import clipboard_output
from soh.__version__ import __version__


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """Sleight of hand CLI commands.

    (+) indicates command group.
    Use the -c flag on most commands to copy output to clipboard
    """
    pass  # pragma: no cover


@click.command(short_help="soh CLI version")
@clipboard_output
def version():
    """Sleight of hand (soh) version."""
    return __version__


cli.add_command(b64, name="b64")
cli.add_command(create, name="create")
cli.add_command(dt, name="dt")
cli.add_command(epoch, name="epoch")
cli.add_command(json_, name="json")
cli.add_command(jwt, name="jwt")
cli.add_command(secret, name="secret")
cli.add_command(serve, name="serve")
cli.add_command(sys, name="sys")
cli.add_command(uuid_, name="uuid")
cli.add_command(version, name="version")


if __name__ == "__main__":
    cli()  # pragma: no cover



"""Epoch time functionality.

Entry point: $ soh epoch [OPTS]
Entry point: $ soh epoch s [OPTS]
Entry point: $ soh epoch ms [OPTS]
Entry point: $ soh epoch us [OPTS]
Entry point: $ soh epoch ns [OPTS]
"""
import time

import arrow
import click

from soh.util import clipboard_output


@click.group(invoke_without_command=False, short_help="+ Epoch times")
def epoch():
    """Epoch times."""
    pass  # pragma: no cover


@epoch.command(short_help="epoch time seconds")
@click.option("-f", "--float", "float_", is_flag=True, default=False, help="return float value")
@clipboard_output
def s(float_):
    """Epoch time in seconds."""
    value = time.time_ns() / 10 ** 9

    if not float_:
        value = int(value)

    return str(value)


@epoch.command(short_help="epoch time milliseconds")
@click.option("-f", "--float", "float_", is_flag=True, default=False, help="return float value")
@clipboard_output
def ms(float_):
    """Epoch time in milliseconds."""
    value = time.time_ns() / 10 ** 6

    if not float_:
        value = int(value)

    return str(value)


@epoch.command(short_help="epoch time microseconds")
@click.option("-f", "--float", "float_", is_flag=True, default=False, help="return float value")
@clipboard_output
def us(float_):
    """Epoch time in microseconds."""
    value = time.time_ns() / 10 ** 3

    if not float_:
        value = int(value)

    return str(value)


@epoch.command(short_help="epoch time nanoseconds")
@click.option("-f", "--float", "float_", is_flag=True, default=False, help="return float value")
@clipboard_output
def ns(float_):
    """Epoch time in nanoseconds."""
    value = time.time_ns()

    if not float_:
        value = int(value)

    return str(value)


@epoch.command(name="from", short_help="epoch time from a timestamp")
@click.option("-f", "--float", "float_", is_flag=True, default=False, help="return float value")
@click.argument("datetime")
@clipboard_output
def from_(float_, datetime):
    """Epoch time from a timestamp (in quotes)."""
    try:
        if float_:
            return str(arrow.get(datetime).float_timestamp)
        else:
            return str(arrow.get(datetime).timestamp)
    except Exception as exc:
        raise click.ClickException(exc)



import os
import uuid

from click.testing import CliRunner
import vcr

from soh.create import gitignore
from soh.create import license_
from soh import create
from soh.util import COPIED_TO_CLIPBOARD_MESSAGE


def test_gitignore(overwrite, language):
    # patch over the gitignore name so we don't overwrite the actual .gitignore
    # use randomness in the name in order to parallelized tests
    create.GITIGNORE_FILE = f"{str(uuid.uuid4())}.gitignore"

    runner = CliRunner()

    # build args
    args = []
    if overwrite is not None:
        args += [overwrite]
    args += [language]

    with vcr.use_cassette(f"tests/cassettes/create_gitignore_{language}.yaml"):
        result = runner.invoke(gitignore, args)

    if language == "invalid_language":
        assert result.exit_code != 0
        return

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    if language == "list":
        assert "python" in output
        return
    else:
        assert os.path.isfile(create.GITIGNORE_FILE)

    # test overwrite functionality
    with vcr.use_cassette(f"tests/cassettes/create_gitignore_{language}.yaml"):
        result = runner.invoke(gitignore, args)

    if overwrite is None:
        assert result.exit_code != 0
    elif overwrite in ("-o", "--overwrite"):
        assert result.exit_code == 0

    assert os.path.isfile(create.GITIGNORE_FILE)

    # cleanup
    os.remove(create.GITIGNORE_FILE)


def test_license(overwrite, value):
    # patch over the license name so we don't overwrite the actual .gitignore
    # use randomness in the name in order to parallelized tests
    create.LICENSE_FILE = f"{str(uuid.uuid4())}_LICENSE.txt"

    runner = CliRunner()

    # build args
    args = []
    if overwrite is not None:
        args += [overwrite]
    args += [value]

    with vcr.use_cassette(f"tests/cassettes/create_license_{value}.yaml"):
        result = runner.invoke(license_, args)

    if value == "invalid_license":
        assert result.exit_code != 0
        return

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    if value == "list":
        assert "mit" in output
        return
    else:
        assert os.path.isfile(create.LICENSE_FILE)

    # test overwrite functionality
    with vcr.use_cassette(f"tests/cassettes/create_license_{value}.yaml"):
        result = runner.invoke(license_, args)

    if overwrite is None:
        assert result.exit_code != 0
    elif overwrite in ("-o", "--overwrite"):
        assert result.exit_code == 0

    assert os.path.isfile(create.LICENSE_FILE)

    # cleanup
    os.remove(create.LICENSE_FILE)



import time

import pytest
from click.testing import CliRunner

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.epoch import from_
from soh.epoch import s
from soh.epoch import ms
from soh.epoch import us
from soh.epoch import ns


FROZEN_TIME_SECONDS = "2019-06-09T04:20:00+00:00"
FROZEN_TIME_EPOCH = "1560054000"
NONSENSE_INPUT = "nonsense"
NONSENSE_OUTPUT = "nonsense"


def test_epoch_s(float_):
    return check_epochs(s, float_, divisor=10 ** 9)


def test_epoch_ms(float_):
    return check_epochs(ms, float_, divisor=10 ** 6)


def test_epoch_us(float_):
    return check_epochs(us, float_, divisor=10 ** 3)


def test_epoch_ns(float_):
    return check_epochs(ns, float_, divisor=1)


@pytest.mark.parametrize(
    "from_input,from_output", [(FROZEN_TIME_SECONDS, FROZEN_TIME_EPOCH), (NONSENSE_INPUT, NONSENSE_OUTPUT)]
)
def test_from(float_, from_input, from_output):
    runner = CliRunner()

    # build args
    args = [from_input]
    if float_ is not None:
        args += [float_]

    result = runner.invoke(from_, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    if from_input == NONSENSE_INPUT:
        assert result.exit_code != 0
    else:
        assert result.exit_code == 0
        assert from_output in output


def check_epochs(func, float_, divisor):
    runner = CliRunner()

    # build args
    args = []
    if float_ is not None:
        args += [float_]

    result = runner.invoke(func, args)

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    assert time.time_ns() / divisor - float(output) < 1 * 10 ** 9 / divisor

    if float_ and func != ns:
        assert "." in output
    else:
        assert "." not in output

    return result



"""System information.

Entry point: $ soh arch
Entry point: $ soh cores
Entry point: $ soh eip
Entry point: $ soh ip
Entry point: $ soh eip
Entry point: $ soh mac
Entry point: $ soh machine
Entry point: $ soh node
Entry point: $ soh proc
Entry point: $ soh sys
Entry point: $ soh sysver
"""
import multiprocessing
import platform
import socket
from uuid import getnode

import click
import requests

from soh.util import clipboard_output
from soh.util import ensure_ok_response


@click.group(invoke_without_command=False, short_help="+ System information")
def sys():
    """System information."""
    pass  # pragma: no cover


@sys.command(name="all", short_help="System information")
@clipboard_output
def all_():
    """System information"""
    return (
        "Arch: " + "; ".join(platform.architecture()) + "\n"
        "Cores: " + str(multiprocessing.cpu_count()) + "\n"
        "External IP: " + get_external_ip() + "\n"
        "Local IP: " + socket.gethostbyname(socket.gethostname()) + "\n"
        "MAC Address: " + ":".join(("%012x" % getnode())[i : i + 2] for i in range(0, 12, 2)) + "\n"
        "Machine type: " + platform.machine() + "\n"
        "Machine name: " + platform.node() + "\n"
        "Platform: " + platform.system() + "\n"
        "Platform version: " + platform.version() + "\n"
        "Processor: " + platform.processor()
    )


@sys.command(short_help="OS architecture")
@clipboard_output
def arch():
    """Platform architecture."""
    return "; ".join(platform.architecture())


@sys.command(short_help="Number of cores")
@clipboard_output
def cores():
    """Number of cores."""
    return str(multiprocessing.cpu_count())


@sys.command(short_help="External IP address")
@clipboard_output
def eip():
    """External IP address."""
    return get_external_ip()


def get_external_ip():
    response = requests.get("https://api6.ipify.org?format=json")
    ensure_ok_response(response, "External request failed.")

    value = response.json()["ip"]

    return value


@sys.command(short_help="Local IP address")
@clipboard_output
def ip():
    """Local IP address."""
    value = socket.gethostbyname(socket.gethostname())
    return str(value)


@sys.command(short_help="Local MAC address")
@click.option("-u", "--upper", is_flag=True, help="use upper case")
@clipboard_output
def mac(upper):
    """MAC address."""
    value = ":".join(("%012x" % getnode())[i : i + 2] for i in range(0, 12, 2))

    if upper:
        value = value.upper()

    return str(value)


@sys.command(short_help="Machine information")
@clipboard_output
def machine():
    """Machine type."""
    return platform.machine()


@sys.command(short_help="Machine name")
@clipboard_output
def node():
    """Node name."""
    return platform.node()


@sys.command(name="platform", short_help="Platform type")
@clipboard_output
def platform_():
    """Platform type."""
    return platform.system()


@sys.command(short_help="Processor information")
@clipboard_output
def proc():
    """Processor."""
    return platform.processor()


@sys.command(short_help="OS version")
@clipboard_output
def version():
    """System release version."""
    return platform.version()



import time

from click.testing import CliRunner
from dateutil.zoneinfo import get_zonefile_instance
from freezegun import freeze_time

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.datetime import time
from soh.datetime import now
from soh.datetime import date
from soh.datetime import from_


FROZEN_TIME = "2019-06-09T04:20:00.123456+00:00"
FROZEN_TIME_SECONDS = "2019-06-09T04:20:00+00:00"
FROZEN_TIME_EPOCH = "1560054000"
FROZEN_TIME_EST = "2019-06-08T23:20:00.123456-05:00"
FROZEN_TIME_EST_SECONDS = "2019-06-08T23:20:00-05:00"


TIMEZONES = sorted(list(get_zonefile_instance().zones))


@freeze_time(FROZEN_TIME)
def test_now(tz_label, tz):
    runner = CliRunner()

    # build args
    args = []
    if tz_label is not None:
        args += [tz_label, tz]

    result = runner.invoke(now, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    if tz_label is not None:
        if tz == "list":
            assert result.exit_code == 0
            assert "EST" in output
        elif tz == "EST":
            assert result.exit_code == 0
            assert output == FROZEN_TIME_EST
        elif tz in TIMEZONES:
            assert result.exit_code == 0
        else:
            assert result.exit_code != 0
    elif tz_label is None:
        assert result.exit_code == 0
        assert output == FROZEN_TIME


@freeze_time(FROZEN_TIME)
def test_today(tz_label, tz):
    runner = CliRunner()

    # build args
    args = []
    if tz_label is not None:
        args += [tz_label, tz]

    result = runner.invoke(date, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    if tz_label is not None:
        if tz == "list":
            assert result.exit_code == 0
            assert "EST" in output
        elif tz == "EST":
            assert result.exit_code == 0
            assert output == FROZEN_TIME_EST[:10]
        elif tz in TIMEZONES:
            assert result.exit_code == 0
        else:
            assert result.exit_code != 0
    elif tz_label is None:
        assert result.exit_code == 0
        assert output == FROZEN_TIME[:10]


@freeze_time(FROZEN_TIME)
def test_time(tz_label, tz):
    runner = CliRunner()

    # build args
    args = []
    if tz_label is not None:
        args += [tz_label, tz]

    result = runner.invoke(time, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    if tz_label is not None:
        if tz == "list":
            assert result.exit_code == 0
            assert "EST" in output
        elif tz == "EST":
            assert result.exit_code == 0
            assert output == FROZEN_TIME_EST[11:]
        elif tz in TIMEZONES:
            assert result.exit_code == 0
        else:
            assert result.exit_code != 0
    elif tz_label is None:
        assert result.exit_code == 0
        assert output == FROZEN_TIME[11:]


def test_from(tz_label, tz):
    runner = CliRunner()

    # build args
    args = [FROZEN_TIME_EPOCH]
    if tz_label is not None:
        args += [tz_label, tz]

    result = runner.invoke(from_, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    if tz_label is not None:
        if tz == "list":
            assert result.exit_code == 0
            assert "EST" in output
        elif tz == "EST":
            print(args)
            assert result.exit_code == 0
            assert output == FROZEN_TIME_EST_SECONDS
        elif tz in TIMEZONES:
            assert result.exit_code == 0
        else:
            assert result.exit_code != 0
    elif tz_label is None:
        assert result.exit_code == 0
        assert output == FROZEN_TIME_SECONDS



import codecs
import string

from click.testing import CliRunner

from soh.secret import AMBIGUOUS_CHARACTERS
from soh.secret import b
from soh.secret import h
from soh.secret import pw
from soh.secret import u
from soh.util import COPIED_TO_CLIPBOARD_MESSAGE


def test_secret_b(nbytes_label, nbytes):
    runner = CliRunner()

    # build args
    args = []
    if nbytes_label is not None:
        args += [nbytes_label, nbytes]

    result = runner.invoke(b, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    # output is a string representation of bytes
    # so we have to unescape it to it's original representation
    if nbytes_label:
        assert len(codecs.decode(output, "unicode_escape")) == nbytes
    else:
        assert len(codecs.decode(output, "unicode_escape")) == 32


def test_secret_h(nbytes_label, nbytes):
    runner = CliRunner()

    # build args
    args = []
    if nbytes_label is not None:
        args += [nbytes_label, nbytes]

    result = runner.invoke(h, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    if nbytes_label:
        assert len(output) == nbytes * 2
    else:
        assert len(output) == 32 * 2


def test_secret_u(nbytes_label, nbytes):
    runner = CliRunner()

    # build args
    args = []
    if nbytes_label is not None:
        args += [nbytes_label, nbytes]

    result = runner.invoke(u, args)

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    # base64 encoding generates ~33% overhead
    if nbytes_label:
        assert len(output) > nbytes * 1.3
        assert len(output) < nbytes * 1.4
    else:
        assert len(output) > 32 * 1.3
        assert len(output) < 32 * 1.4


def test_secret_pw(qty_num_label, qty_num, qty_sym_label, qty_sym, length_label, length, allow_ambiguous, protect):
    runner = CliRunner()

    # build args
    args = []
    if qty_num_label is not None:
        args += [qty_num_label, qty_num]
    if qty_sym_label is not None:
        args += [qty_sym_label, qty_sym]
    if length_label is not None:
        args += [length_label, length]
    if allow_ambiguous is not None:
        args += [allow_ambiguous]
    if protect is not None:
        args += [protect]

    result = runner.invoke(pw, args)

    num = qty_num if qty_num_label is not None else 8
    sym = qty_sym if qty_sym_label is not None else 4
    lth = length if length_label is not None else 32

    if num + sym > lth:
        assert result.exit_code != 0
        return
    else:
        assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")

    if protect is not None:
        assert len(output) == 0
        return

    if length_label is not None:
        assert len(output) == length

    num_count = 0
    sym_count = 0
    for c in output:
        if c in string.digits:
            num_count += 1
        elif c in string.punctuation:
            sym_count += 1

    assert num_count == num
    assert sym_count == sym

    if allow_ambiguous is None:
        for c in AMBIGUOUS_CHARACTERS:
            assert c not in output



from click.testing import CliRunner

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE


def compare(func, args, compare_to=None, clip=None):
    runner = CliRunner()

    result = runner.invoke(func, args)

    assert result.exit_code == 0

    if compare_to is not None:
        output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
        assert output == compare_to

    return result



import signal
import time

from subprocess import PIPE
from subprocess import Popen
from subprocess import TimeoutExpired

import requests


def test_serve(port_label):
    # Use a different port for each port label
    # because the OS takes a long time to free
    # up the ports when trying to reuse
    ports = {"-p": 8081, "--port": 8082}
    port = ports.get(port_label, None)

    # build args
    args = []
    if port_label is not None:
        args += [port_label, str(port)]
    command = ["soh", "serve"]
    if len(args) > 0:
        command += args

    # launch in a subprocess
    process = Popen(command, stdout=PIPE)
    # let the server spin up
    time.sleep(3)

    # make a request for the readme file
    p = str(port if port_label is not None else 8080)
    host = f"http://localhost:{p}"
    url = f"{host}/README.rst"
    print(url)
    try:
        response = requests.get(url)
    except requests.exceptions.ConnectionError:
        process.kill()
        raise

    # terminate the grab stdout to verify
    process.send_signal(signal.SIGINT)
    try:
        outs, _ = process.communicate(timeout=3)
    except TimeoutExpired:
        process.kill()
        outs, _ = process.communicate()

    process.kill()

    assert response.status_code == 200

    assert host in outs.decode()



import json

from click.testing import CliRunner

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.tokens import jwt


def test_jwt(indent_label, indent, jwt_sample):
    runner = CliRunner()
    # build args
    args = []
    if indent_label is not None:
        args += [indent_label, indent]
    else:
        indent = 4
    args += [jwt_sample]

    result = runner.invoke(jwt, args)

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0]
    print(output)

    if indent_label is not None:
        for row in output.split("\n"):
            print(row, len(row), len(row.lstrip()))
            assert (len(row) - len(row.lstrip())) % indent == 0

    header = {"alg": "HS256", "typ": "JWT"}
    payload = {"sub": "1234567890", "name": "John Doe", "iat": 1516239022}
    signature = "SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c"

    assert json.dumps(header, indent=indent) in output
    assert json.dumps(payload, indent=indent) in output
    assert signature in output



"""Base64 CLI functionality.

Entry point: $ soh b64d [OPTS] <text>
Entry point: $ soh b64e [OPTS] <text>
"""
import base64

import click
import pyperclip

from soh.util import clipboard_output


def apply_padding(text):
    return text + "=" * (len(text) % 4)


@click.group(invoke_without_command=False, short_help="+ Base64 operations")
def b64():
    """Base64 decoding and encoding."""
    pass  # pragma: no cover


@b64.command(short_help="Decode base64 strings")
@click.option("-p", "--pad", is_flag=True, default=False, show_default=True, help="try to apply missing padding")
@click.argument("text", required=False)
@clipboard_output
def d(text, pad):
    """Decode base64 to text.

    If `text` argument is missing, uses clipboard contents.
    """
    if text is None:  # pragma: no cover
        click.echo("No argument passed. Using clipboard contents...")
        text = pyperclip.paste()
    if pad:
        text = apply_padding(text)
    try:
        return base64.b64decode(text.encode("utf-8")).decode("utf-8")
    except Exception as exc:  # pragma: no cover
        raise click.ClickException("Unable to decode: " + str(exc))


@b64.command(short_help="Encode strings to base64")
@click.argument("text", required=False)
@clipboard_output
def e(text):
    """Encode text to base64.

    If `text` argument is missing, uses clipboard contents.
    """
    if text is None:  # pragma: no cover
        click.echo("No argument passed. Using clipboard contents...")
        text = pyperclip.paste()
    try:
        return base64.b64encode(text.encode("utf-8")).decode("utf-8")
    except Exception as exc:  # pragma: no cover
        raise click.ClickException("Unable to encode: " + str(exc))



"""JSON printing CLI functionality.

Entry point: $ soh json [OPTS] <text>
"""
import json

import click
import pyperclip

from soh.util import clipboard_output


@click.command(short_help="JSON printing")
@click.option("-i", "--indent", default=4, help="indent quantity")
@click.option("-a", "--ascii", "ascii_", is_flag=True, default=False, help="ensure ascii")
@click.argument("text", required=False)
@clipboard_output
def json_(indent, text, ascii_):
    """JSON printing.

    Use single quotes around the JSON text.
    """
    if text is None:  # pragma: no cover
        click.echo("No argument passed. Using clipboard contents...")
        text = pyperclip.paste()
    try:
        return json.dumps(json.loads(text), indent=indent, ensure_ascii=ascii_)
    except json.decoder.JSONDecodeError:
        raise click.ClickException("Unable to parse JSON.")



from functools import wraps

import click
from click.testing import CliRunner
import pyperclip
import pytest
import requests
import vcr

from soh.util import clipboard_output
from soh.util import ensure_ok_response
from soh.util import COPIED_TO_CLIPBOARD_MESSAGE


CLIPBOARD_TEST_VALUE = "qweoridvlasna;ebvaubv"
FAILURE_MESSAGE = "fail message"


def check_clipboard_output(func):
    """Decorator to test clipboard output behavior.

    Doesn't work when running pytest xdist since parallelisation doesn't make sense
    with a single clipboard.
    """

    @wraps(func)
    def clipboard_checker(*args, **kwargs):
        pyperclip.copy(CLIPBOARD_TEST_VALUE)
        clip = kwargs.get("clip", None)

        result = func(*args, **kwargs)

        # only check clipboard results on success
        if result.exit_code != 0:
            return

        if clip is not None:
            assert COPIED_TO_CLIPBOARD_MESSAGE in result.output
            assert pyperclip.paste() != CLIPBOARD_TEST_VALUE
        else:
            assert COPIED_TO_CLIPBOARD_MESSAGE not in result.output
            assert pyperclip.paste() == CLIPBOARD_TEST_VALUE

    return clipboard_checker


@click.command(short_help="dummy command")
@clipboard_output
def dummy():
    return "something"


# @check_clipboard_output
def check_dummy(clip):
    runner = CliRunner()
    args = []
    if clip is not None:
        args += [clip]
    result = runner.invoke(dummy, args)
    assert result.exit_code == 0
    return result


def test_dummy():
    """Test the clipboard behavior serially."""
    for clip in [None, "-c", "--clip"]:
        pyperclip.copy(CLIPBOARD_TEST_VALUE)
        result = check_dummy(clip)
        if clip is not None:
            assert COPIED_TO_CLIPBOARD_MESSAGE in result.output
            assert pyperclip.paste() != CLIPBOARD_TEST_VALUE
        else:
            assert COPIED_TO_CLIPBOARD_MESSAGE not in result.output
            assert pyperclip.paste() == CLIPBOARD_TEST_VALUE


def test_ensure_ok_response():
    with vcr.use_cassette("tests/cassettes/util_success_response.yaml"):
        response = requests.get("https://httpbin.org/status/200")
        ensure_ok_response(response, FAILURE_MESSAGE)

    with vcr.use_cassette("tests/cassettes/util_fail_response.yaml"):
        response = requests.get("https://httpbin.org/status/500")
        with pytest.raises(click.ClickException) as exc:
            ensure_ok_response(response, FAILURE_MESSAGE)
            assert repr(exc) == FAILURE_MESSAGE



from .common import compare
from soh.cli import version
from soh.__version__ import __version__


# @check_clipboard_output
def test_version():  # , clip):
    args = []
    compare(version, args, __version__)



"""UUID CLI functionality.

Entry point: $ soh uuid [OPTS]
"""
import uuid

import click

from soh.util import clipboard_output


@click.command(short_help="Generate UUIDs")
@click.option("-v", "--version", "uuid_version", default=4, show_default=True, help="uuid version {1, 3, 4, 5}")
@click.option("-ns", "--namespace", "uuid_namespace", help="namespace (for v3, v5) {dns, url, oid, x500}")
@click.option("-n", "--name", "uuid_name", default="", help="name (for v3, v5)")
@click.option("-u", "--upper", is_flag=True, help="use upper case")
@clipboard_output
def uuid_(uuid_version, uuid_namespace, uuid_name, upper):
    """Generate UUIDs."""
    if uuid_namespace is not None:
        if uuid_version not in (3, 5):
            raise click.ClickException("Namespaces only valid for versions {3, 5}")
        namespaces = {
            "dns": uuid.NAMESPACE_DNS,
            "url": uuid.NAMESPACE_URL,
            "oid": uuid.NAMESPACE_OID,
            "x500": uuid.NAMESPACE_X500,
        }
        try:
            ns = namespaces[uuid_namespace.lower()]
        except KeyError:
            raise click.ClickException("Invalid namespace, must be in {dns, url, oid, x500}")
    elif uuid_version in (3, 5):
        raise click.ClickException(f"Namespace must be provided for version {uuid_version}")

    if uuid_version == 1:
        value = str(uuid.uuid1())
    elif uuid_version == 3:
        value = str(uuid.uuid3(ns, uuid_name))
    elif uuid_version == 4:
        value = str(uuid.uuid4())
    elif uuid_version == 5:
        value = str(uuid.uuid5(ns, uuid_name))
    else:
        raise click.ClickException("Invalid version")

    if upper:
        value = value.upper()

    return value



"""Simple HTTP fileserver.

Entry point: $ soh serve [OPTS]
"""
from http.server import SimpleHTTPRequestHandler
import socketserver

import click


# TODO consider specifying the path


@click.command(short_help="Simple http server at current directory")
@click.option("-p", "--port", default=8080, show_default=True, help="port")
def serve(port):
    """Simple HTTP Server."""
    with socketserver.TCPServer(("", port), SimpleHTTPRequestHandler) as httpd:
        print("Serving at http://localhost:" + str(port) + " ... Press CTRL-C to quit.")
        httpd.serve_forever()



"""File creation helper functionality.

Entry point: $ soh create gitignore [OPTS] <text>
Entry point: $ soh create license [OPTS] <text>
"""
import os

import click
import requests

from soh.util import clipboard_output
from soh.util import ensure_ok_response


GITIGNORE_FILE = ".gitignore"
LICENSE_FILE = "LICENSE.txt"


@click.group(invoke_without_command=False, short_help="+ Create files")
def create():
    """Create files."""
    pass  # pragma: no cover


@create.command(short_help="Create a .gitignore file from github.com/git/gitignore")
@click.option("-o", "--overwrite", is_flag=True, default=False, show_default=True, help="overwrite existing .gitignore")
@click.argument("language")
@clipboard_output
def gitignore(language, overwrite):
    """Create a .gitignore file.

    Use `list` to get a list of available languages.
    """
    response = requests.get("https://api.github.com/repos/github/gitignore/contents")
    ensure_ok_response(response, "Fetching .gitignore files failed.")

    contents = response.json()

    if language == "list":
        languages = []
        for element in contents:
            filename_split = element["name"].split(".")
            if filename_split[-1] == "gitignore":
                languages.append(filename_split[0].lower())
        return " * " + "\n * ".join(languages)

    download_url = None
    for element in contents:
        if language.lower() == element["name"].split(".")[0].lower():
            download_url = element["download_url"]

    if download_url is None:
        raise click.ClickException("Language " + language + " could not be found.")

    response = requests.get(download_url)
    ensure_ok_response(response, "Fetching .gitignore file failed.")

    file_exists = os.path.isfile(GITIGNORE_FILE)

    if file_exists and overwrite is False:
        raise click.ClickException(GITIGNORE_FILE + " file already exists. Delete it or use -o to overwrite.")

    with open(GITIGNORE_FILE, "w") as f:
        f.write(response.text)

    return "File " + GITIGNORE_FILE + " was created."


@create.command(name="license", short_help="Create a LICENSE.txt file from github.com/github/choosealicense.com")
@click.option("-o", "--overwrite", is_flag=True, default=False, show_default=True, help="overwrite existing .gitignore")
@click.argument("value")
@clipboard_output
def license_(value, overwrite):
    """Create a LICENSE.txt file.

    Use `list` to get a list of available licenses.
    """
    response = requests.get("https://api.github.com/repos/github/choosealicense.com/contents/_licenses?ref=gh-pages")
    ensure_ok_response(response, "Fetching license files failed.")

    contents = response.json()

    if value == "list":
        licenses = []
        for element in contents:
            filename_split = element["name"].split(".")
            if filename_split[-1] == "txt":
                licenses.append(filename_split[0].lower())
        return " * " + "\n * ".join(licenses)

    download_url = None
    for element in contents:
        if value.lower() == element["name"].split(".")[0].lower():
            download_url = element["download_url"]

    if download_url is None:
        raise click.ClickException("License " + value + " could not be found.")

    response = requests.get(download_url)
    ensure_ok_response(response, "Fetching LICENSE file failed.")

    file_exists = os.path.isfile(LICENSE_FILE)

    if file_exists and overwrite is False:
        raise click.ClickException(LICENSE_FILE + " file already exists. Delete it or use -o to overwrite.")

    with open(LICENSE_FILE, "w") as f:
        # files have metadata in header with --- separators
        f.write(response.text.split("---")[-1].strip())

    return "File " + LICENSE_FILE + " was created."



import json

from click.testing import CliRunner

from soh.util import COPIED_TO_CLIPBOARD_MESSAGE
from soh.json import json_


def test_json(indent_label, indent, ascii_, json_text):
    runner = CliRunner()

    # build args
    args = []
    if indent_label is not None:
        args += [indent_label, indent]
    if ascii_ is not None:
        args += [ascii_]
    args += [json_text]

    result = runner.invoke(json_, args)

    try:
        json.loads(json_text)
    except json.decoder.JSONDecodeError:
        assert result.exit_code != 0
        return

    assert result.exit_code == 0

    output = result.output.rsplit(COPIED_TO_CLIPBOARD_MESSAGE)[0].replace("\n", "")
    if indent_label is not None:
        for row in output.split("\n"):
            assert (len(row) - len(row.lstrip())) % indent == 0
    if ascii_:
        assert output != json.dumps(json.loads(json_text), indent=indent, ensure_ascii=False)
    assert output == json.dumps(json.loads(json_text), indent=indent, ensure_ascii=ascii_)



"""Setup."""
from os import path
from setuptools import setup
from setuptools import find_packages


here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

# import __version__ attributes
about = {}
with open(path.join(here, "soh", "__version__.py")) as f:
    exec(f.read(), about)

setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=long_description,
    url=about["__url__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    license=about["__license__"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    keywords="soh cli tool",
    packages=find_packages(),
    install_requires=["arrow", "click", "pyperclip", "requests"],
    zip_safe=False,
    entry_points={"console_scripts": ["soh=soh.cli:cli"]},
)



"""Version information."""
__title__ = "soh"
__description__ = "Sleight of hand (soh) CLI tool."
__url__ = "https://github.com/crflynn/soh"
__version__ = "0.1.0"
__author__ = "Christopher Flynn"
__author_email__ = "crf204@gmail.com"
__license__ = "MIT"
__copyright__ = "Copyright 2019"
__docs_copyright__ = "2019"



"""UTC Datetime information.

Entry point: $ soh now [OPTS]
Entry point: $ soh time [OPTS]
Entry point: $ soh today [OPTS]
"""
import arrow
from arrow.parser import ParserError
from dateutil.zoneinfo import get_zonefile_instance
import click

from soh.util import clipboard_output


@click.group(invoke_without_command=False, short_help="+ Datetimes")
def dt():
    """Base64 decoding and encoding."""
    pass  # pragma: no cover


def get_tz_time(time_, tz):
    if tz == "list":
        return "\n".join(sorted(list(get_zonefile_instance().zones)))
    if tz is not None:
        try:
            the_time = time_.to(tz)
        except ParserError:
            raise click.ClickException("Unknown timezone: " + tz)
    else:
        the_time = time_
    return the_time


@dt.command(short_help="UTC now")
@click.option("-tz", show_default=True, help="timezone")
@clipboard_output
def now(tz):
    """Current datetime."""
    return str(get_tz_time(arrow.utcnow(), tz))


@dt.command(short_help="UTC time")
@click.option("-tz", show_default=True, help="timezone")
@clipboard_output
def time(tz):
    """Current time."""
    if tz == "list":
        return str(get_tz_time(arrow.utcnow(), tz))
    else:
        return str(get_tz_time(arrow.utcnow(), tz))[11:]


@dt.command(short_help="UTC date")
@click.option("-tz", show_default=True, help="timezone")
@clipboard_output
def date(tz):
    """Current date."""
    if tz == "list":
        return str(get_tz_time(arrow.utcnow(), tz))
    else:
        return str(get_tz_time(arrow.utcnow(), tz).date())


@dt.command(name="from", short_help="Datetime from epoch seconds")
@click.option("-tz", show_default=True, help="timezone")
@click.argument("epoch")
@clipboard_output
def from_(tz, epoch):
    """Datetime from epoch seconds."""
    if tz == "list":
        return str(get_tz_time(arrow.get(epoch), tz))
    else:
        return str(get_tz_time(arrow.get(epoch), tz))



"""JWT CLI functionality.

Entry point: $ soh jwt [OPTS] <text>
"""
import base64
import json

import click
import pyperclip

from soh.util import clipboard_output


def b64decode(segment):
    return base64.urlsafe_b64decode(segment + "=" * (len(segment) % 4)).decode("utf-8")


def segment_to_dict(segment):
    return json.loads(b64decode(segment))


@click.command(short_help="Display JWT contents")
@click.option("-i", "--indent", default=4, show_default=True, help="json indent")
@click.argument("token", required=False)
@clipboard_output
def jwt(indent, token):
    if token is None:  # pragma: no cover
        click.echo("No argument passed. Using clipboard contents...")
        token = pyperclip.paste()
    try:
        header, payload, signature = token.split(".")
        output = "header = " + json.dumps(segment_to_dict(header), indent=indent)
        output += "\n"
        output += "payload = " + json.dumps(segment_to_dict(payload), indent=indent)
        output += "\n"
        output += 'signature = "' + signature + '"'
        return output
    except Exception as exc:  # pragma: no cover
        raise click.ClickException(exc)


# TODO paseto
